@@ -20,6 +20,7 @@ import functools
 import gettext
 import glob
 import hashlib
+import tempfile
 import json
 import locale
 import logging
@@ -900,7 +901,7 @@ def rerun_lock(name, folder=None, logger=_logger):
     return decorator
 
 
-def append_to_os_paths(bindir, usesyspath=True):
+def append_to_os_paths(bindir, usesyspath=True, inmemonly=False):
     """
     On macOS, PATH update will only take effect after calling `source ~/.bash_profile` directly in shell. It won't work 
     """
@@ -930,23 +931,24 @@ def append_to_os_paths(bindir, usesyspath=True):
     os.environ[path_var] += os.pathsep + bindir
 
 
-def prepend_to_os_paths(bindir, usesyspath=True):
+def prepend_to_os_paths(bindir, usesyspath=True, inmemonly=False):
+    path_var = 'Path' if platform.system() == 'Windows' else 'PATH'
+    if inmemonly:
+        os.environ[path_var] = bindir + os.pathsep + os.environ[path_var]
+        return
     if platform.system() == 'Windows':
         import winreg
         root_key = winreg.HKEY_LOCAL_MACHINE if usesyspath else winreg.HKEY_CURRENT_USER
         sub_key = r'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment' if usesyspath else r'Environment'
-        path_var = 'Path'
-        separator = ';'
         with winreg.ConnectRegistry(None, root_key):
             with winreg.OpenKey(root_key, sub_key, 0, winreg.KEY_ALL_ACCESS) as key:
                 env_paths, _ = winreg.QueryValueEx(key, path_var)
-                matches = [path for path in env_paths.split(separator) if path.lower() == bindir.lower()]
+                matches = [path for path in env_paths.split(os.pathsep) if path.lower() == bindir.lower()]
                 if matches:
                     return
-                env_paths = f'{bindir}{separator}' + env_paths
+                env_paths = f'{bindir}{os.pathsep}' + env_paths
                 winreg.SetValueEx(key, path_var, 0, winreg.REG_EXPAND_SZ, env_paths)
     else:
-        path_var = 'PATH'
         cfg_file = os.path.expanduser('~/.bash_profile') if platform.system() == 'Darwin' else os.path.expanduser('~/.bashrc')
         if bindir in os.environ[path_var]:
             return
@@ -958,24 +960,25 @@ def prepend_to_os_paths(bindir, usesyspath=True):
     os.environ[path_var] = bindir + os.pathsep + os.environ[path_var]
 
 
-def remove_from_os_paths(bindir, usesyspath=True):
+def remove_from_os_paths(bindir, usesyspath=True, inmemonly=False):
+    path_var = 'Path' if platform.system() == 'Windows' else 'PATH'
+    if inmemonly:
+        os.environ[path_var] = os.environ[path_var].replace(bindir, '')
+        return
     if platform.system() == 'Windows':
         import winreg
         root_key = winreg.HKEY_LOCAL_MACHINE if usesyspath else winreg.HKEY_CURRENT_USER
         sub_key = r'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment' if usesyspath else r'Environment'
-        separator = ';'
-        path_var = 'Path'
         with winreg.ConnectRegistry(None, root_key):
             with winreg.OpenKey(root_key, sub_key, 0, winreg.KEY_ALL_ACCESS) as key:
                 env_paths, _ = winreg.QueryValueEx(key, path_var)
-                matches = [path for path in env_paths.split(separator) if path.lower() == bindir.lower()]
+                matches = [path for path in env_paths.split(os.pathsep) if path.lower() == bindir.lower()]
                 if not matches:
                     return
-                keepers = [path for path in env_paths.split(separator) if path.lower() != bindir.lower()]
-                env_paths = separator.join(keepers)
+                keepers = [path for path in env_paths.split(os.pathsep) if path.lower() != bindir.lower()]
+                env_paths = os.pathsep.join(keepers)
                 winreg.SetValueEx(key, path_var, 0, winreg.REG_EXPAND_SZ, env_paths)
     else:
-        path_var = 'PATH'
         cfg_file = os.path.expanduser('~/.bash_profile') if platform.system() == 'Darwin' else os.path.expanduser('~/.bashrc')
         if bindir not in os.environ[path_var]:
             return
@@ -1404,11 +1407,21 @@ def unzip_dir(srcball, destpardir):
     assume srcball has a top-level folder "product".
     unzip to destpardir/product.
     """
-    untar_option = '-xf' if platform.system() == 'Windows' else '-xzf'
-    os.makedirs(destpardir, exist_ok=True)
-    cmd = ['tar', untar_option, srcball, '-C', destpardir]
-    run_cmd(cmd, destpardir)
-    
+    ext = osp.splitext(srcball)[1]
+    if ext == '.zip':
+        untar_option = '-xf' if platform.system() == 'Windows' else '-xzf'
+        os.makedirs(destpardir, exist_ok=True)
+        cmd = ['tar', untar_option, srcball, '-C', destpardir]
+        run_cmd(cmd, destpardir)
+        return
+    elif ext == '.xz':
+        import tarfile
+        os.makedirs(destpardir, exist_ok=True)
+        with tarfile.open(srcball) as f:
+            f.extractall(destpardir)
+        return
+    raise NotImplementedError(f'Unsupported zip type: {ext}')
+
 
 def duplicate_dir(srcdir, dstdir):
     def _dup_dir_posix(sdir, ddir):
@@ -1724,6 +1737,86 @@ def save_text(path, text, toappend=False):
         fp.write(text)
 
 
+def remove_duplication(mylist):
+    return list(dict.fromkeys(mylist))
+
+
+def install_by_macports(pkg, ver=None, lazybin=None):
+    """
+    Homebrew has the top priority.
+    Macports only overrides in memory on demand.
+    """
+    os_paths = os.environ['PATH']
+    prepend_to_os_paths('/opt/local/sbin', inmemonly=True)
+    prepend_to_os_paths('/opt/local/bin', inmemonly=True)
+    if lazybin and (exe := shutil.which(lazybin)):
+        print(f'Found binary: {exe}, and skipped installing package: {pkg}')
+        return
+    run_cmd(['sudo', 'port', 'install', pkg])
+    os.environ['PATH'] = os_paths
+
+
+def uninstall_by_macports(pkg, ver=None):
+    """
+    Homebrew has the top priority.
+    Macports only overrides in memory on demand.
+    """
+    os_paths = os.environ['PATH']
+    prepend_to_os_paths('/opt/local/sbin', inmemonly=True)
+    prepend_to_os_paths('/opt/local/bin', inmemonly=True)
+    run_cmd(['sudo', 'port', 'uninstall', pkg])
+    os.environ['PATH'] = os_paths
+
+
+def install_by_homebrew(pkg, ver=None, lazybin=None):
+    if lazybin and (exe := shutil.which(lazybin)):
+        print(f'Found binary: {exe}, and skipped installing package: {pkg}')
+        return
+    run_cmd(['brew', 'install', pkg])
+
+
+def uninstall_by_homebrew(pkg, ver=None):
+    run_cmd(['brew', 'remove', pkg])
+
+
+class Autotools:
+    def __init__(self, pkgroot, logger=None):
+        self.pkgRoot = pkgroot
+        self.logger = logger
+
+    def init(self):
+        install_by_homebrew('autoconf', lazybin='/usr/local/bin/autoreconf')
+        install_by_homebrew('automake', lazybin='/usr/local/bin/automake')
+
+    def autogen(self):
+        run_cmd(['./autogen.sh'], cwd=self.pkgRoot, logger=self.logger)
+
+    def configure(self, flags):
+        run_cmd(['./configure'] + flags, cwd=self.pkgRoot, logger=self.logger)
+
+    def make_install(self, njobs=8):
+        run_cmd(['make', f'-j{njobs}'], cwd=self.pkgRoot, logger=self.logger)
+        run_cmd(['make', 'install'], cwd=self.pkgRoot, logger=self.logger)
+
+    def make_clean(self):
+        run_cmd(['make', 'clean'], cwd=self.pkgRoot, logger=self.logger)
+
+
+def build_apple_iconset(master, iconset):
+    iconset_dir = osp.join(f'/{get_platform_tmp_dir()}/icons/icon.iconset')
+    os.makedirs(iconset_dir, exist_ok=True)
+    sizes = ('16', '16@2x', '32', '32@2x', '128', '128@2x', '256', '256@2x', '512')
+    for sz in sizes:
+        comps = sz.split('@')
+        sz = comps[0]
+        actual_size = str(2*int(sz)) if (has_sep := len(comps) > 1) else sz
+        suffix = f'{sz}x{sz}@2x' if has_sep else f'{sz}x{sz}'
+        run_cmd(['sips', '-z', actual_size, actual_size, master, '--out', osp.abspath(f'{iconset_dir}/icon_{suffix}.png')])
+    copy_file(master, osp.abspath(f'{iconset_dir}/icon_512x512@2x.png'))
+    run_cmd(['iconutil', '-c', 'icns', iconset_dir, '-o', iconset])
+    shutil.rmtree(iconset_dir, ignore_errors=True)
+
+
 def _test():
     pass
 
